@@ -31,6 +31,7 @@ int emptyline(char *);
 int iscomment(char *);
 int isnumber(char *);
 int isserver(char *);
+int isvaliduserhost(char *);
 int isansi(char *);
 int isaction(char *);
 int isctcp(char *);
@@ -342,6 +343,25 @@ int isserver(char *line)
 	}
 }
 
+int isvaliduserhost(char *line)
+{
+	int c;
+	int c2;
+	char *ptr;
+	char *ptr2;
+
+        if((ptr = index(line, '@')) == NULL) {
+		return 0;
+	}
+	ptr2 = ptr + strlen(line);
+	c = *(ptr - 1);
+	c2 = *(ptr + 1);
+	if(ptr == line || ptr == ptr2 || !isgraph(c) || !isgraph(c2)) {
+		return 0;
+	}
+	return 1;
+}
+
 int parse(char *from, int pos, char *delim, char *to, int to_length, int fixed)
 {
         int i;
