@@ -29,13 +29,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "misc.h"
 
 /* global (to all) variables */
-struct state *mystate;
+
 int do_console;
 int do_debug;
 int do_output;
 int do_tempoutput;
 
 char salt_chars[] = { "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" };
+char timed_str[WRITE_BUFFER_LENGTH + 1];
+
+struct state *mystate;
 
 /* global (to oer.c) variables */
 int do_signals;
@@ -225,12 +228,6 @@ int main(int argc, char *argv[])
 		if((mystate->last_actions + 1) >= mystate->now) {
 			continue;
 		}
-		switch(processmmodes()) {
-		case OER_PROCESSMMODES_ERR_WRITE:
-			mystate->reconnect = OER_RECONNECT_ERROR;
-			reconnect();
-			break;
-		}
 		switch(processtimeds()) {
 		case OER_PROCESSTIMEDS_ERR_WRITE:
 			mystate->reconnect = OER_RECONNECT_ERROR;
