@@ -160,7 +160,7 @@ void parseirc(char *serveroutput)
 				/* isoper */
 				for(this = mystate->channels; this != NULL; this = this->next) {
 					if(isonchan(this->name, nick)) {
-						changeuser(this->name, nick, 1, -1, -1);
+						changeuser(this->name, nick, 1, -1, -1, -1);
 					}
 				}
 			}
@@ -361,7 +361,7 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		oer_debug(OER_DEBUG_INFO, "parseirc(484)->nick %s on %s is a IRC operator\n", nick, channel);
-		changeuser(channel, nick, 1, -1, -1);
+		changeuser(channel, nick, 1, -1, -1, -1);
 		return;
 	}
 	if(!strcasecmp(command, "513")) {
@@ -445,13 +445,16 @@ void parseirc(char *serveroutput)
 			}
 		}
 		if(amiop(channel) && !kicked && haschanflags(channel, "v") && hasuserflags(channel, userhost, "v")) {
-			addnewmm(channel, mystate->now,  "+v", nick);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +v %s\n", channel, nick);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		}
 		if(amiop(channel) && !kicked && haschanflags(channel, "O") && isopa(userhost)) {
-			addnewmm(channel, mystate->now, "+o", nick);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, nick);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		}
 		if(amiop(channel) && !kicked && haschanflags(channel, "o") && isop(channel, userhost)) {
-			addnewmm(channel, mystate->now, "+o", nick);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, nick);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		}
 		if(!kicked && haschanflags(channel, "A")) {
 			sendadverts(channel, nick, userhost);
@@ -515,8 +518,8 @@ void parseirc(char *serveroutput)
 			}
 			if(usersonchan(this->name) == 1 && !amiop(this->name)) {
 				oer_debug(OER_DEBUG_INFO, "parseirc(QUIT)->cycling %s to gain ops\n", this->name);
-				snprintf(outstring2, WRITE_BUFFER_LENGTH, "PART %s\n", this->name);
-				addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring2, NULL, NULL, NULL);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s\n", this->name);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 			}
 		}
 		listchanusers();
@@ -542,8 +545,8 @@ void parseirc(char *serveroutput)
 		}
 		if(usersonchan(channel) == 1 && !amiop(channel)) {
 			oer_debug(OER_DEBUG_INFO, "parseirc(PART)->cycling %s to gain ops\n", channel);
-			snprintf(outstring, STRINGLEN, "PART %s\n", channel);
-			addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s\n", channel);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		}
 		listchanusers();
 		return;
@@ -613,8 +616,8 @@ void parseirc(char *serveroutput)
 		}
 		if(usersonchan(channel) == 1 && !amiop(channel)) {
 			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->cycling %s to gain ops\n", channel);
-			snprintf(outstring2, WRITE_BUFFER_LENGTH, "PART %s\n", channel);
-			addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring2, NULL, NULL, NULL);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s\n", channel);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		}
 		listchanusers();
 		return;
@@ -668,7 +671,7 @@ void parseirc(char *serveroutput)
 			modetype = nthmode(mode, i);
 			switch(modetype) {
 			case OER_NTHMODE_OP:
-				changeuser(channel, target, -1, 1, -1);
+				changeuser(channel, target, -1, 1, -1, -1);
 				if(isme(target)) {
 					iamop(channel, 1);
 					channelsync(channel);
@@ -683,21 +686,31 @@ void parseirc(char *serveroutput)
 						continue;
 					}
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, nick, userhost);
-					addnewmm(channel, mystate->now, "-o", target);
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, target);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 				}
 				break;
 			case OER_NTHMODE_DOP:
-				changeuser(channel, target, -1, 0, -1);
+				changeuser(channel, target, -1, 0, -1, -1);
 				if(isme(target)) {
 					iamop(channel, 0);
 				}
+				if(isq(nick)) {
+					/* allow Q to freely de-op */
+					continue;
+				}
+				if(haschanflags(channel, "D") && isallowedop(channel, target)) {
+					/* re-op */
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, target);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
+				}
 				deops++;
 				break;
 			case OER_NTHMODE_VOICE:
-				changeuser(channel, target, -1, -1, 1);
+				changeuser(channel, target, -1, -1, 1, -1);
 				break;
 			case OER_NTHMODE_DVOICE:
-				changeuser(channel, target, -1, -1, 0);
+				changeuser(channel, target, -1, -1, 0, -1);
 				break;
 			case OER_NTHMODE_BAN:
 				snprintf(outstring, STRINGLEN, "%s!%s@%s", (mystate->use_alt_nick) ? mystate->altnick : mystate->nick, mystate->user, mystate->host);
@@ -710,10 +723,11 @@ void parseirc(char *serveroutput)
 					if(amiop(channel) && haschanflags(channel, "b")) {
 						/* i am being banned, first kick then unban */
 						if(!isme(nick) && !isserver(nick)) {
-							snprintf(outstring, STRINGLEN, "KICK %s %s :do NOT ban %s, that's me!\n", channel, nick, target);
-							addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+							snprintf(timed_str, WRITE_BUFFER_LENGTH, "KICK %s %s :do NOT ban %s, that's me!\n", channel, nick, target);
+							timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 						}
-						addnewmm(channel, mystate->now, "-b", target);
+						snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -b %s\n", channel, target);
+						timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 					}
 				}
 				bans++;
@@ -726,7 +740,8 @@ void parseirc(char *serveroutput)
 					}
 					/* re-set the ban */
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", nick, channel, target);
-					addnewmm(channel, mystate->now, "+b", target);
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +b %s\n", channel, target);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 					if(!isserver(nick)) {
 						snprintf(outstring, STRINGLEN, "do NOT remove %s ban from %s, it is in my permban list", target, channel);
 						sendreply(nick, 0, 3, 0, outstring);
@@ -746,7 +761,8 @@ void parseirc(char *serveroutput)
 					continue;
 				}
 				if(amiop(channel) && haschanflags(channel, "m") && !isserver(nick)) {
-					addnewmm(channel, mystate->now, "-o", nick);
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, nick);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 					sendreply(nick, 0, 3, 0, "do NOT deop >3 users at a time");
 					deops = 0;
 					bans = 0;
@@ -1086,8 +1102,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                         return;
                 }
 		delalltopics(channel);
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "TOPIC %s\n", channel);
-		addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+		snprintf(timed_str, WRITE_BUFFER_LENGTH, "TOPIC %s\n", channel);
+		timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		return;
 	case OER_WHICHCOMMAND_TOPIC_REFRESH:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->TOPIC_REFRESH\n");
@@ -1275,7 +1291,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!strlen(restptr)) {
 			/* the messager wants to be opped */
 			if(!isopnow(channel, nick)) {
-				addnewmm(channel, mystate->now, "+o", nick);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, nick);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 			return;
 		}
@@ -1291,7 +1308,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 				if(haschanflags(channel, "u") && !isallowedop(channel, p1)) {
 					continue;
 				}
-				addnewmm(channel, mystate->now, "+o", p1);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, p1);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 		}
 		return;
@@ -1309,7 +1327,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!strlen(restptr)) {
 			/* the messager wants to be de-opped */
 			if(isopnow(channel, nick)) {
-				addnewmm(channel, mystate->now, "-o", nick);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, nick);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 			return;
 		}
@@ -1322,7 +1341,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			}
 			ppos = nppos;
 			if(isopnow(channel, p1)) {
-				addnewmm(channel, mystate->now, "-o", p1);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, p1);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 		}
 		return;
@@ -1340,7 +1360,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!strlen(restptr)) {
 			/* the messager wants to be voiced */
 			if(!isvoicenow(channel, nick)) {
-				addnewmm(channel, mystate->now, "+v", nick);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +v %s\n", channel, nick);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 			return;
 		}
@@ -1353,7 +1374,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			}
 			ppos = nppos;
 			if(!isvoicenow(channel, p1)) {
-				addnewmm(channel, mystate->now, "+v", p1);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +v %s\n", channel, p1);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 		}
 		return;
@@ -1463,8 +1485,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			/* this makes no sense */
 			return;
 		}
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "INVITE %s %s\n", nick, channel);
-		addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+		snprintf(timed_str, WRITE_BUFFER_LENGTH, "INVITE %s %s\n", nick, channel);
+		timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		return;
 	case OER_WHICHCOMMAND_MASSMESSAGE:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->MASSMESSAGE\n");
@@ -1495,7 +1517,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		}
 		if(!strlen(restptr + ppos)) {
 			/* only a mode line, no target */
-			addnewmm(channel, mystate->now, p1, "");
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s %s\n", channel, p1);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			return;
 		}
 		/* process all modes */
@@ -1508,7 +1531,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			ppos = nppos;
 			getnthmode(p1, counter, modeline);
 			oer_debug(OER_DEBUG_INFO, "parsecommand->mode %s %s %s\n", channel, modeline, p2);
-			addnewmm(channel, mystate->now, modeline, p2);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s %s %s\n", channel, modeline, p2);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			counter++;
 		}
 		return;
@@ -1724,8 +1748,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			return;
 		}
 		if(strlen(restptr) > 0) {
-			snprintf(outstring, WRITE_BUFFER_LENGTH, "%s\n", restptr);
-                        addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "%s\n", restptr);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		}
 		return;
 	case OER_WHICHCOMMAND_LAST:
@@ -1798,6 +1822,20 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		}
 		wall(channel, nick, userhost, restptr);
 		return;
+	case OER_WHICHCOMMAND_CLONECHANNEL:
+		oer_debug(OER_DEBUG_INFO, "parsecommand->CLONECHANNEL\n");
+                if(!isopa(userhost)) {
+                        return;
+                }
+		if(!strlen(restptr)) {
+			return;
+		}
+		if(!clonechannel(channel, restptr)) {
+			return;
+		}
+		snprintf(outstring, WRITE_BUFFER_LENGTH, "cloned channel %s to %s", channel, restptr);
+		sendreply((tochan) ? channel : nick, tochan, 0, 0, outstring);
+		return;
         case OER_WHICHCOMMAND_INVALID:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->invalid or malformed command %s\n", command);
 		break;
